from floraconcierge.client.types import Object


class Submit(Object):
    """
    :type order: Object
    :type validation: Result
    """

    def __init__(self, *args, **kwargs):
        self.order = None
        self.validation = None

        super(Submit, self).__init__(*args, **kwargs)



from floraconcierge.client.types import Object


class Code(Object):

    """
    :type code: str
    :type id: int
    """

    def __init__(self, *args, **kwargs):
        self.code = None
        self.discount = None
        self.id = None

        super(Code, self).__init__(*args, **kwargs)



from importlib import import_module
import sys

from django.conf import settings
from django.http import Http404
from django.utils import six

from floraconcierge import shortcuts
from floraconcierge.apiauth.models import User
from floraconcierge.cache import get_request_cache, new_request_cache
from floraconcierge.client import ApiClient
from floraconcierge.errors import ResultObjectNotFoundError, MiddlewareError


def import_string(dotted_path):
    """
    Import a dotted module path and return the attribute/class designated by the
    last name in the path. Raise ImportError if the import failed.

    Helper function from django 1.7
    """
    try:
        module_path, class_name = dotted_path.rsplit('.', 1)
    except ValueError:
        msg = "%s doesn't look like a module path" % dotted_path
        six.reraise(ImportError, ImportError(msg), sys.exc_info()[2])

    try:
        # noinspection PyUnboundLocalVariable
        module = import_module(module_path)
    except ImportError, e:
        six.reraise(ImportError, ImportError('%s. Tried to import %s' % (e.message, module_path)), sys.exc_info()[2])

    try:
        # noinspection PyUnboundLocalVariable
        return getattr(module, class_name)
    except AttributeError:
        msg = 'Module "%s" does not define a "%s" attribute/class' % (
            dotted_path, class_name)
        six.reraise(ImportError, ImportError(msg), sys.exc_info()[2])


def initialize_apiclient(request):
    err_pattern = 'floraconcierge.middleware.ApiClientMiddleware requires to be set settings.%s'

    api_id = getattr(settings, 'FLORACONCIERGE_API_ID', None)
    if not api_id:
        raise ValueError(err_pattern % 'FLORACONCIERGE_API_ID')

    secret = getattr(settings, 'FLORACONCIERGE_API_SECRET', None)
    if not secret:
        raise ValueError(err_pattern % 'FLORACONCIERGE_API_SECRET')

    client = ApiClient(api_id, secret)

    sess_env = request.session.get("__floraconcierge_api_env", None)
    restored = False
    if sess_env:
        client.env = sess_env
        restored = True

    init_env = getattr(settings, 'FLORACONCIERGE_API_INIT_ENV', None)
    if init_env:
        init_env = import_string(init_env)
        env = init_env(client, request, restored)
        if env:
            client.env = env

    return client


class ApiClientMiddleware(object):
    def process_request(self, request):
        init = getattr(settings, 'FLORACONCIERGE_API_INIT_CLIENT', None)
        if init:
            init = import_string(init)
            client = init(request)
        else:
            client = initialize_apiclient(request)

        request.api = client
        request.apienv = client.env

        shortcuts.activate(client)

    def process_response(self, request, response):
        if hasattr(request, 'apienv'):
            # Support api user authentication
            if isinstance(request.user, User) and request.user.info:
                request.apienv.user_auth_key = request.user.info.auth.auth_key
            else:
                request.apienv.user_auth_key = ''

            request.session['__floraconcierge_api_env'] = request.apienv

        return response


class ApiObjectNotFound404(object):
    def process_exception(self, request, exception):
        if isinstance(exception, ResultObjectNotFoundError):
            raise Http404


_cache_cls = None


class RequestCacheMiddleware(object):
    @staticmethod
    def cache_cls():
        global _cache_cls

        if not _cache_cls:
            cls = getattr(settings, 'FLORACONCIERGE_CACHE_CLASS', None)
            if not cls:
                raise MiddlewareError('Please define FLORACONCIERGE_CACHE_CLASS in settings for RequestCacheMiddleware')

            _cache_cls = import_string(cls)

        return _cache_cls

    def process_request(self, request):
        try:
            cache = get_request_cache()
        except MiddlewareError:
            cache = new_request_cache(self.cache_cls())

        cache.clear()
        request.cache = cache



from floraconcierge.shortcuts import get_apiclient


def get_cities_list(country_id=None, region_id=None, limit=None, offset=None):
    return get_apiclient().services.geo.get_cities_list(
        country_id=country_id,
        region_id=region_id,
        limit=limit,
        offset=offset)


def get_city_by_id(id):
    return get_apiclient().services.geo.get_city_by_id(id)


def get_continent_by_id(id):
    return get_apiclient().services.geo.get_continent_by_id(id)


def get_continents_list(limit=None, offset=None):
    return get_apiclient().services.geo.get_continents_list(
        limit=limit,
        offset=offset)


def get_countries_list(parent_id=None, limit=None, offset=None):
    return get_apiclient().services.geo.get_countries_list(
        parent_id=parent_id,
        limit=limit,
        offset=offset)


def get_country_by_id(id):
    return get_apiclient().services.geo.get_country_by_id(id)


def get_region_by_id(id):
    return get_apiclient().services.geo.get_region_by_id(id)


def get_regions_list(country_id=None, limit=None, offset=None):
    return get_apiclient().services.geo.get_regions_list(
        country_id=country_id,
        limit=limit,
        offset=offset)


def get_sub_continent_by_id(id):
    return get_apiclient().services.geo.get_sub_continent_by_id(id)


def get_subcontinents_list(parent_id, limit=None, offset=None):
    return get_apiclient().services.geo.get_subcontinents_list(
        parent_id,
        limit=limit,
        offset=offset)


def get_user_ip_info():
    return get_apiclient().services.geo.get_user_ip_info()



import math

from floraconcierge.client.types import Object


DEFAULT_PRODUCT_IMAGES_LIST = {
    'default': 'https://www.florastatic.net/img/products/%s_250.jpg',
    'small': 'https://www.florastatic.net/img/products/%s_100.jpg',
    'middle': 'https://www.florastatic.net/img/products/%s_250.jpg',
    'big': 'https://www.florastatic.net/img/products/%s_500.jpg',
    'thumb_100': 'https://www.florastatic.net/img/products/%s_100.jpg',
    'thumb_250': 'https://www.florastatic.net/img/products/%s_250.jpg',
    'thumb_500': 'https://www.florastatic.net/img/products/%s_500.jpg',
}
_product_images_urls = None


def get_product_images_urls():
    global _product_images_urls

    if not _product_images_urls:
        from django.conf import settings
        from django.core.exceptions import ImproperlyConfigured

        try:
            urls = getattr(settings, 'FLORACONCIERGE_PRODUCT_IMAGES', DEFAULT_PRODUCT_IMAGES_LIST)
        except ImproperlyConfigured:
            urls = DEFAULT_PRODUCT_IMAGES_LIST

        if not isinstance(urls, dict):
            raise ImproperlyConfigured('FLORACONCIERGE_PRODUCT_IMAGES configuration must be dict')

        _product_images_urls = urls

    return _product_images_urls


class Currency(Object):
    def __init__(self, *args, **kwargs):
        super(Currency, self).__init__(*args, **kwargs)

        try:
            self.rounding = float(self.rounding)
        except (ValueError, TypeError):
            self.rounding = .0

    def round(self, value, decimals=0):
        if not self.rounding:
            multiplier = float(10 ** decimals)

            return int(value * multiplier) / multiplier
        else:
            if self.rounding_direction == 'desc':
                return int(self.rounding * math.ceil(value / self.rounding))
            else:
                return int(self.rounding * math.floor(value / self.rounding))

    def _format(self, value):
        return self.format % value

    def convert(self, value):
        return self._format(self.convert_float(value))

    def convert_float(self, value):
        return self.round(float(self.usdvalue) * float(value), 2)


class Product(Object):
    def __init__(self, *args, **kwargs):
        super(Product, self).__init__(*args, **kwargs)

        urls = {}
        if self.url:
            for k, v in get_product_images_urls().iteritems():
                urls[k] = v % self.id

        self.urls = urls



from floraconcierge.client import Service
# noinspection PyShadowingBuiltins


class Shop(Service):

    def get_categories_list(self, parent_id=None):
        """
        :rtype : list of Category|Collection
        """
        return self._callapi("shop/categories", get={"parent_id": parent_id})

    def get_category_by_id(self, id):
        """
        :rtype : Category
        """
        return self._callapi("shop/category", get={"id": id})

    def get_currencies_list(self, order_id=None):
        """
        :rtype : list of Currency|Collection
        """
        return self._callapi("shop/currencies", get={"order_id": order_id})

    def get_currency_by_id(self, id, order_id=None):
        """
        :rtype : Currency
        """
        return self._callapi(
            "shop/currency",
            get={
                "id": id,
                "order_id": order_id})

    def get_gallery(self):
        """
        :rtype : Collection
        """
        return self._callapi("shop/gallery")

    def calculate_order(self, postObject):
        """
        :rtype : Order
        """
        return self._callapi("shop/order/calculate", post=postObject)

    def get_order_by_id(self, id):
        """
        :rtype : Order
        """
        return self._callapi("shop/order/get", get={"id": id})

    def get_last_delivered(self):
        """
        :rtype : Collection
        """
        return self._callapi("shop/order/lastdelivered")

    def get_user_orders(self, is_draft=None):
        """
        :rtype : list of Order|Collection
        """
        return self._callapi("shop/order/my", get={"is_draft": is_draft})

    def get_order_reasons_text_by_id(self, id):
        """
        :rtype : Text
        """
        return self._callapi("shop/order/reason/text", get={"id": id})

    def get_order_reasons_texts(self, parent_id=None):
        """
        :rtype : list of Text|Collection
        """
        return self._callapi(
            "shop/order/reason/texts",
            get={
                "parent_id": parent_id})

    def get_order_reason_by_id(self, id):
        """
        :rtype : Reason
        """
        return self._callapi("shop/order/reason", get={"id": id})

    def get_order_reasons(self):
        """
        :rtype : list of Reason|Collection
        """
        return self._callapi("shop/order/reasons")

    def get_order_status(self, id):
        """
        :rtype : list
        """
        return self._callapi("shop/order/status", get={"id": id})

    def submit_order_phone(self, postObject):
        """
        :rtype : Submit
        """
        return self._callapi("shop/order/submit/phone", post=postObject)

    def submit_order_request(self, postObject):
        """
        :rtype : int
        """
        return self._callapi("shop/order/submit/request", post=postObject)

    def submit_order(self, postObject):
        """
        :rtype : Submit
        """
        return self._callapi("shop/order/submit", post=postObject)

    def submit_order_ticket(self, id, key, message):
        """
        :rtype : bool
        """
        return self._callapi(
            "shop/order/tickets/submit",
            get={
                "id": id,
                "key": key,
                "message": message})

    def get_order_tickets(self, id, key):
        """
        :rtype : list of list|Collection
        """
        return self._callapi("shop/order/tickets", get={"id": id, "key": key})

    def get_payment_system_by_id(self, id):
        """
        :rtype : Paysystem
        """
        return self._callapi("shop/paysystem", get={"id": id})

    def get_payment_systems_list(self):
        """
        :rtype : Collection|list of Paysystem
        """
        return self._callapi("shop/paysystems")

    def get_product_by_id(self, id):
        """
        :rtype : Product
        """
        return self._callapi("shop/product", get={"id": id})

    def get_products_of_day(self):
        """
        :rtype : list of Product|Collection
        """
        return self._callapi("shop/products/day")

    def get_featured_products(
            self,
            city_id=None,
            country_id=None,
            product_of_day=None):
        """
        :rtype : list of Product|Collection
        """
        return self._callapi(
            "shop/products/featured",
            get={
                "city_id": city_id,
                "country_id": country_id,
                "product_of_day": product_of_day})

    def get_parent_product_by_id(self, id):
        """
        :rtype : Product
        """
        return self._callapi("shop/products/parent", get={"id": id})

    def search_products(self, query):
        """
        :rtype : list of Product|Collection
        """
        return self._callapi("shop/products/search", get={"query": query})

    def get_products_list(
            self,
            category=None,
            products=None,
            ingredient=None,
            random=None):
        """
        :rtype : list of Product|Collection
        """
        return self._callapi(
            "shop/products",
            get={
                "category": category,
                "products": products,
                "ingredient": ingredient,
                "random": random})

    def check_promo_code(self, code):
        """
        :rtype : Code
        """
        return self._callapi("shop/promo/check", get={"code": code})

    def get_review_by_id(self, id):
        """
        :rtype : Review
        """
        return self._callapi("shop/review", get={"id": id})

    def submit_review(self, postObject):
        """
        :rtype : int
        """
        return self._callapi("shop/reviews/submit", post=postObject)

    def get_reviews(
            self,
            limit=None,
            offset=None,
            product_id=None,
            after_id=None):
        """
        :rtype : Collection|list of Review
        """
        return self._callapi(
            "shop/reviews",
            get={
                "limit": limit,
                "offset": offset,
                "product_id": product_id,
                "after_id": after_id})

    def get_cash_salons_cities(self, country_id):
        """
        :rtype : City|Collection
        """
        return self._callapi(
            "shop/salons/cities",
            get={
                "country_id": country_id})

    def get_cash_salons_countries(self):
        """
        :rtype : Country
        """
        return self._callapi("shop/salons/countries")

    def get_cash_salons_by_id(self, salon_id):
        """
        :rtype : Salon
        """
        return self._callapi("shop/salons/salon", get={"salon_id": salon_id})

    def get_cash_salons_by_city(self, city_id):
        """
        :rtype : list of Salon|Collection
        """
        return self._callapi("shop/salons", get={"city_id": city_id})
# noinspection PyShadowingBuiltins


class Users(Service):

    def address_delete(self, id):
        """
        :rtype : int
        """
        return self._callapi("users/address/delete", get={"id": id})

    def address_list(self):
        """
        :rtype : list of Address|Collection
        """
        return self._callapi("users/address/list")

    def address_process(self, postObject):
        """
        :rtype : Address
        """
        return self._callapi("users/address/process", post=postObject)

    def calendar_delete(self, id):
        """
        :rtype : int
        """
        return self._callapi("users/calendar/delete", get={"id": id})

    def calendar_list(self):
        """
        :rtype : list of Calendar|Collection
        """
        return self._callapi("users/calendar/list")

    def calendar_process(self, postObject):
        """
        :rtype : Calendar
        """
        return self._callapi("users/calendar/process", post=postObject)

    def check_password(self, email, password):
        """
        :rtype : bool
        """
        return self._callapi(
            "users/checkpassword",
            get={
                "email": email,
                "password": password})

    def exists(self, email):
        """
        :rtype : bool
        """
        return self._callapi("users/exists", get={"email": email})

    def get_logged_user(self, info=None):
        """
        :rtype : Stdclass|Null|User
        """
        return self._callapi("users/logged", get={"info": info})

    def login(self, email, password=None, remember=None, checkip=None):
        """
        :rtype : User
        """
        return self._callapi(
            "users/login",
            get={
                "email": email,
                "password": password,
                "remember": remember,
                "checkip": checkip})

    def logout(self):
        """
        :rtype : bool
        """
        return self._callapi("users/logout")

    def registration(self, email):
        """
        :rtype : bool
        """
        return self._callapi("users/registration", get={"email": email})

    def reset_password(self, email):
        """
        :rtype : User
        """
        return self._callapi("users/resetpassword", get={"email": email})

    def social_list(self):
        """
        :rtype : Stdclass
        """
        return self._callapi("users/social/list")

    def social_login(self, token):
        """
        :rtype : User
        """
        return self._callapi("users/social/login", get={"token": token})

    def social_related(self):
        """
        :rtype : Stdclass
        """
        return self._callapi("users/social/related")

    def social_signin(self, service, redirect_success, redirect_fail):
        """
        :rtype : str
        """
        return self._callapi(
            "users/social/signin",
            get={
                "service": service,
                "redirect_success": redirect_success,
                "redirect_fail": redirect_fail})

    def update_info(
            self,
            password=None,
            name=None,
            phone=None,
            country=None,
            city=None,
            birthday=None,
            flowers=None,
            feed_news=None,
            gender=None):
        """
        :rtype : User
        """
        return self._callapi(
            "users/update",
            get={
                "password": password,
                "name": name,
                "phone": phone,
                "country": country,
                "city": city,
                "birthday": birthday,
                "flowers": flowers,
                "feed_news": feed_news,
                "gender": gender})
# noinspection PyShadowingBuiltins


class Img(Service):

    def get_watermark(self, img_name):
        """
        :rtype : Stdclass
        """
        return self._callapi("img/watermark", get={"img_name": img_name})
# noinspection PyShadowingBuiltins


class Phone(Service):

    def search(self, phone):
        """
        :rtype : Stdclass
        """
        return self._callapi("phone/search", get={"phone": phone})
# noinspection PyShadowingBuiltins


class Invoices(Service):

    def get_invoice_by_key(self, id):
        """
        :rtype : Invoice
        """
        return self._callapi("invoices/invoice", get={"id": id})

    def get_order_by_key(self, id):
        """
        :rtype : Order
        """
        return self._callapi("invoices/order", get={"id": id})

    def process_invoice_by_id(self, id, paysystem_id, amount, postObject):
        """
        :rtype : bool
        """
        return self._callapi(
            "invoices/process",
            get={
                "id": id,
                "paysystem_id": paysystem_id,
                "amount": amount},
            post=postObject)
# noinspection PyShadowingBuiltins


class System(Service):

    def send_feedback(self, postObject):
        """
        :rtype : bool
        """
        return self._callapi("system/feedback", post=postObject)

    def get_language_by_id(self, id):
        """
        :rtype : Language
        """
        return self._callapi("system/language", get={"id": id})

    def get_languages_list(self, limit=None, offset=None):
        """
        :rtype : Collection|list of Language
        """
        return self._callapi(
            "system/languages",
            get={
                "limit": limit,
                "offset": offset})

    def get_page_by_id(self, id):
        """
        :rtype : Page
        """
        return self._callapi("system/page", get={"id": id})

    def get_pages_list(self, category=None):
        """
        :rtype : list of Page|Collection
        """
        return self._callapi("system/pages", get={"category": category})

    def get_snippet_by_id(self, id):
        """
        :rtype : Snippet
        """
        return self._callapi("system/snippet", get={"id": id})
# noinspection PyShadowingBuiltins


class Auth(Service):

    def set_result(self, token_id, is_success, data):
        """
        :rtype : Signin
        """
        return self._callapi(
            "auth/result",
            get={
                "token_id": token_id,
                "is_success": is_success,
                "data": data})

    def get_token(self, id):
        """
        :rtype : Signin
        """
        return self._callapi("auth/token", get={"id": id})
# noinspection PyShadowingBuiltins


class Discount(Service):

    def email(self, email):
        """
        :rtype : bool
        """
        return self._callapi("discount/email", get={"email": email})
# noinspection PyShadowingBuiltins


class Partner(Service):

    def history_all_ref_codes(self, after_id=None):
        """
        :rtype : list of Stdclass|list
        """
        return self._callapi("partner/ref/all", get={"after_id": after_id})

    def check_ref_code(self, code):
        """
        :rtype : bool
        """
        return self._callapi("partner/ref/check", get={"code": code})

    def generate_ref_code(self):
        """
        :rtype : str
        """
        return self._callapi("partner/ref/generate")

    def history_ref_code(self, code, limit=None, offset=None):
        """
        :rtype : list of Stdclass|list
        """
        return self._callapi(
            "partner/ref/history",
            get={
                "code": code,
                "limit": limit,
                "offset": offset})
# noinspection PyShadowingBuiltins


class Validation(Service):

    def validate_order(self, postObject):
        """
        :rtype : Result
        """
        return self._callapi("validation/order", post=postObject)
# noinspection PyShadowingBuiltins


class Geo(Service):

    def get_cities_list(
            self,
            country_id=None,
            region_id=None,
            limit=None,
            offset=None):
        """
        :rtype : list of City|Collection
        """
        return self._callapi(
            "geo/cities",
            get={
                "country_id": country_id,
                "region_id": region_id,
                "limit": limit,
                "offset": offset})

    def get_city_by_id(self, id):
        """
        :rtype : City
        """
        return self._callapi("geo/city", get={"id": id})

    def get_continent_by_id(self, id):
        """
        :rtype : Continent
        """
        return self._callapi("geo/continent", get={"id": id})

    def get_continents_list(self, limit=None, offset=None):
        """
        :rtype : list of Continent|Collection
        """
        return self._callapi(
            "geo/continents",
            get={
                "limit": limit,
                "offset": offset})

    def get_countries_list(self, parent_id=None, limit=None, offset=None):
        """
        :rtype : list of Country|Collection
        """
        return self._callapi(
            "geo/countries",
            get={
                "parent_id": parent_id,
                "limit": limit,
                "offset": offset})

    def get_country_by_id(self, id):
        """
        :rtype : Country
        """
        return self._callapi("geo/country", get={"id": id})

    def get_region_by_id(self, id):
        """
        :rtype : Region
        """
        return self._callapi("geo/region", get={"id": id})

    def get_regions_list(self, country_id=None, limit=None, offset=None):
        """
        :rtype : list of Country|Collection
        """
        return self._callapi(
            "geo/regions",
            get={
                "country_id": country_id,
                "limit": limit,
                "offset": offset})

    def get_sub_continent_by_id(self, id):
        """
        :rtype : Subcontinent
        """
        return self._callapi("geo/subcontinent", get={"id": id})

    def get_subcontinents_list(self, parent_id, limit=None, offset=None):
        """
        :rtype : list of Subcontinent|Collection
        """
        return self._callapi(
            "geo/subcontinents",
            get={
                "parent_id": parent_id,
                "limit": limit,
                "offset": offset})

    def get_user_ip_info(self):
        """
        :rtype : Stdclass
        """
        return self._callapi("geo/useripinfo")


class Manager(object):

    def __init__(self, apiclient):
        self.shop = Shop(apiclient)
        self.users = Users(apiclient)
        self.img = Img(apiclient)
        self.phone = Phone(apiclient)
        self.invoices = Invoices(apiclient)
        self.system = System(apiclient)
        self.auth = Auth(apiclient)
        self.discount = Discount(apiclient)
        self.partner = Partner(apiclient)
        self.validation = Validation(apiclient)
        self.geo = Geo(apiclient)



from floraconcierge.client.types import Object


class Invoice(Object):

    """
    :type id: int
    :type paid: int
    """

    def __init__(self, *args, **kwargs):
        self.cost = None
        self.created = None
        self.id = None
        self.is_hold = None
        self.is_paid = None
        self.is_test = None
        self.key = None
        self.order_id = None
        self.paid = None
        self.paysystem_id = None
        self.price = None
        self.url = None

        super(Invoice, self).__init__(*args, **kwargs)


class Reason(Object):

    """
    :type id: int
    :type name: str
    """

    def __init__(self, *args, **kwargs):
        self.id = None
        self.name = None

        super(Reason, self).__init__(*args, **kwargs)



from floraconcierge.client.types import Object


class Result(Object):
    """
    :type errors: Object
    :type validated: Object
    """

    def __init__(self, *args, **kwargs):
        self.errors = None
        self.validated = None

        super(Result, self).__init__(*args, **kwargs)

    @property
    def is_valid(self):
        return not self.errors

    def __iter__(self):
        return iter(self.errors)



import hashlib
import pickle
from threading import currentThread

from django.core.cache import cache
from django.core.cache.backends.base import DEFAULT_TIMEOUT
from django.core.cache.backends.locmem import LocMemCache
from django.utils import translation

from floraconcierge.errors import MiddlewareError


_request_cache = {}
_installed_middleware = False


def get_request_cache():
    try:
        return _request_cache[currentThread()]
    except KeyError:
        raise MiddlewareError('floraconcierge.middleware.RequestCacheMiddleware not loaded')


def new_request_cache(cachecls):
    _request_cache[currentThread()] = cache = cachecls()

    return cache


def cached(func=None, timeout=DEFAULT_TIMEOUT, *args, **kwargs):
    def _make_key(func, args, kwargs):
        lang = translation.get_language()

        if args and hasattr(args[0], 'get') and hasattr(args[0], 'add'):
            data = [lang, func.__name__, args[0].__class__.__name__, pickle.dumps((args[1:], kwargs))]
        else:
            data = [lang, func.__name__, pickle.dumps((args[1:], kwargs))]

        return hashlib.md5("%".join(data)).hexdigest()

    def inner(func):
        def wrapped(*args, **kwargs):
            key = _make_key(func, args, kwargs)

            if args and hasattr(args[0], 'get') and hasattr(args[0], 'add'):
                obj = args[0]

                if key in obj:
                    return obj.get(key)
                else:
                    result = func(*args, **kwargs)
                    obj.add(key, result, timeout=timeout)

                    return result
            else:
                result = cache.get(key)
                if not result:
                    result = func(*args, **kwargs)
                    cache.set(key, result, timeout=timeout)

                return result

        return wrapped

    if func is None:
        # @cached(...)
        return inner
    elif callable(func):
        # @cached
        return inner(func)
    else:
        raise ValueError('Bad @cached decorator usage')


class RequestCache(LocMemCache):
    def __init__(self):
        name = 'requestcache@%i' % hash(currentThread())
        super(RequestCache, self).__init__(name, {})



from importlib import import_module
from pprint import pformat
from datetime import datetime

from pytz import timezone

from floraconcierge import errors


MIN_INT_TO_DATETIME = 500000000
# If int value is greater then this constant, object will have field ${field}_date which will be datetime type.
# Date 05.11.1985. Too big value for usual usage and fine date :)


def module_name_from_class(cls):
    class_name = ('floraconcierge.mapping.%s' % cls.replace("_", '.').lower()).split('.')
    module, name = ".".join(class_name[:-1]), class_name[-1].capitalize()

    return module, name


def from_dict(obj):
    """
    Creates floraconcierge api result object instance
    @param obj: dict
    @return: Object
    """
    if '_className' in obj and obj['_className']:
        module, name = module_name_from_class(obj['_className'])

        try:
            m = import_module(module)
            cls = getattr(m, name)
        except (ImportError, AttributeError):
            errtpl = '%s.%s for api "%s" mapping not found'
            raise errors.ModelMappingNotFoundError(errtpl % (module, name, obj['_className']))

        return cls(**obj)

    return Object(**obj)


class Object(object):
    def __init__(self, *args, **kwargs):
        super(Object, self).__init__()

        if '_className' in kwargs:
            del kwargs['_className']

        extra = {}
        for k, v in kwargs.iteritems():
            if not isinstance(v, (int, float)):
                try:
                    v = int(v)
                except (ValueError, TypeError):
                    try:
                        v = float(v)
                    except (ValueError, TypeError):
                        pass

            magic_date = '%s_date' % k
            if isinstance(v, (int, float)) and v > MIN_INT_TO_DATETIME and magic_date not in kwargs:
                try:
                    extra[magic_date] = datetime.fromtimestamp(v, timezone('Europe/Moscow'))
                except:
                    extra[magic_date] = None

            kwargs[k] = v

        self.__dict__.update(kwargs)
        self.__dict__.update(extra)

    def __repr__(self):
        if not hasattr(self, '_repr_cache'):
            setattr(self, '_repr_cache', '<%s %s>' % (self.__class__.__name__, pformat(self.__dict__)))

        return getattr(self, '_repr_cache')

    def __iter__(self):
        if hasattr(super(Object, self), '__iter__'):
            # noinspection PyUnresolvedReferences
            return super(Object, self).__iter__()

        return self.__dict__.copy().iteritems()

    def todict(self):
        data = self.__dict__
        data.pop('_repr_cache', None)

        return data

    def datetime(self, unixtime):
        return datetime.fromtimestamp(int(unixtime))



class ApiError(Exception):
    def __init__(self, *args, **kwargs):
        self.result = kwargs.pop('result', None)

        super(ApiError, self).__init__(*args, **kwargs)


class HTTPError(ApiError):
    def __init__(self, base_http_error, *args, **kwargs):
        self.base_error = base_http_error

        super(HTTPError, self).__init__(*args, **kwargs)


class MalformedJSONResultError(ApiError):
    pass


class ModelMappingNotFoundError(ApiError):
    pass


class MiddlewareError(ApiError):
    pass


class ResultError(ApiError):
    pass


class ResultDefaultError(ApiError):
    pass


class ResultBadArgumentsError(ApiError):
    pass


class ResultInvalidApiIDValueError(ApiError):
    pass


class ResultInvalidSignValueError(ApiError):
    pass


class ResultApplicationNotFoundError(ApiError):
    pass


class ResultBadSignatureError(ApiError):
    pass


class ResultApiNotFoundError(ApiError):
    pass


class ResultObjectNotFoundError(ApiError):
    pass


class ResultInvalidParameterValueError(ApiError):
    pass


class ResultUserAuthorizationRequiredError(ApiError):
    pass


def get_result_error(result, error):
    errors = {
        0xffff: ResultDefaultError,
        0x01: ResultBadArgumentsError,
        0x02: ResultInvalidApiIDValueError,
        0x03: ResultInvalidSignValueError,
        0x04: ResultApplicationNotFoundError,
        0x05: ResultBadSignatureError,
        0x06: ResultApiNotFoundError,
        0x07: ResultObjectNotFoundError,
        0x08: ResultInvalidParameterValueError,
        0x10: ResultUserAuthorizationRequiredError
    }

    cls = errors.get(error.code, ResultDefaultError)

    return cls(error.message, result=result)



from django.contrib.auth.backends import ModelBackend
from django.core.exceptions import ValidationError
from django.core.validators import validate_email

from floraconcierge.apiauth.models import User
from floraconcierge.errors import ResultDefaultError
from floraconcierge.shortcuts import get_apiclient
from floraconcierge.shortcuts.users import login, get_logged_user, social_login


class FloraConciergeBackend(ModelBackend):
    def authenticate(self, username=None, password=None, **kwargs):
        email = None
        try:
            validate_email(username)
            email = username
        except ValidationError:
            pass

        if email:
            try:
                return User.from_api_user(login(username, password))
            except ResultDefaultError, e:
                raise ValidationError(e.message)

    def get_user(self, user_id):
        try:
            if not get_apiclient().env.user_auth_key:
                return None

            user = get_logged_user()
            if user:
                return User.from_api_user(user)
        except ResultDefaultError:
            return None


class FloraConciergeOauthBackend(FloraConciergeBackend):
    def authenticate(self, flora_token=None, **kwargs):
        try:
            return User.from_api_user(social_login(flora_token))
        except ResultDefaultError, e:
            raise ValidationError(e.message)



from django.contrib import admin
from django.utils.translation import gettext_lazy as _

from floraconcierge.apiauth import models


class User(admin.ModelAdmin):
    fieldsets = (
        (_('Personal info'), {'fields': ('email',)}),
        (_('Permissions'), {'fields': ('is_active', 'is_staff', 'is_superuser',
                                       'groups', 'user_permissions')}),
        (_('Important dates'), {'fields': ('last_login', 'date_joined')}),
    )
    list_display = ('email', 'is_active', 'is_staff', 'is_superuser', 'last_login')
    list_filter = ('is_staff', 'is_superuser', 'is_active', 'groups')
    search_fields = ('email',)
    ordering = ('email',)
    filter_horizontal = ('groups', 'user_permissions',)


admin.site.register(models.User, User)



from floraconcierge.client.types import Object


class Subcontinent(Object):

    """
    :type id: int
    :type iso: str
    :type name: str
    :type slug: str
    """

    def __init__(self, *args, **kwargs):
        self.continent_id = None
        self.id = None
        self.iso = None
        self.name = None
        self.price = None
        self.slug = None

        super(Subcontinent, self).__init__(*args, **kwargs)


class Continent(Object):

    """
    :type id: int
    :type iso: str
    :type name: str
    :type slug: str
    """

    def __init__(self, *args, **kwargs):
        self.id = None
        self.iso = None
        self.name = None
        self.slug = None

        super(Continent, self).__init__(*args, **kwargs)


class City(Object):

    """
    :type id: int
    :type name: str
    :type slug: str
    """

    def __init__(self, *args, **kwargs):
        self.country_id = None
        self.deliver = None
        self.gmt = None
        self.id = None
        self.large = None
        self.lat = None
        self.lng = None
        self.name = None
        self.name_original = None
        self.region_id = None
        self.slug = None
        self.timezone = None
        self.title_unique = None

        super(City, self).__init__(*args, **kwargs)


class Country(Object):

    """
    :type id: int
    :type iso: str
    :type name: str
    :type slug: str
    """

    def __init__(self, *args, **kwargs):
        self.fips = None
        self.id = None
        self.is_cis = None
        self.iso = None
        self.name = None
        self.populations = None
        self.slug = None
        self.subcontinent_id = None
        self.work = None

        super(Country, self).__init__(*args, **kwargs)


class Region(Object):

    """
    :type id: int
    :type iso: str
    :type name: str
    :type slug: str
    """

    def __init__(self, *args, **kwargs):
        self.country_id = None
        self.fips = None
        self.id = None
        self.iso = None
        self.name = None
        self.name_origin = None
        self.populations = None
        self.slug = None

        super(Region, self).__init__(*args, **kwargs)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('email', models.EmailField(unique=True, max_length=75)),
                ('date_joined', models.DateTimeField(default=django.utils.timezone.now, verbose_name='date joined')),
                ('last_login', models.DateTimeField(default=django.utils.timezone.now, verbose_name='last login')),
                ('is_staff', models.BooleanField(default=False, help_text='Designates whether the user can log into this admin site.', verbose_name='staff status')),
                ('is_active', models.BooleanField(default=True, help_text='Designates whether this user should be treated as active. Unselect this instead of deleting accounts.', verbose_name='active')),
                ('groups', models.ManyToManyField(related_query_name='user', related_name='user_set', to='auth.Group', blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of his/her group.', verbose_name='groups')),
                ('user_permissions', models.ManyToManyField(related_query_name='user', related_name='user_set', to='auth.Permission', blank=True, help_text='Specific permissions for this user.', verbose_name='user permissions')),
            ],
            options={
                'swappable': 'AUTH_USER_MODEL',
                'verbose_name': 'user',
                'verbose_name_plural': 'users',
            },
            bases=(models.Model,),
        ),
    ]



import collections


class CaseInsensitiveDict(collections.MutableMapping):
    """
    https://github.com/kennethreitz/requests/blob/v1.2.3/requests/structures.py#L37

A case-insensitive ``dict``-like object.

Implements all methods and operations of
``collections.MutableMapping`` as well as dict's ``copy``. Also
provides ``lower_items``.

All keys are expected to be strings. The structure remembers the
case of the last key to be set, and ``iter(instance)``,
``keys()``, ``items()``, ``iterkeys()``, and ``iteritems()``
will contain case-sensitive keys. However, querying and contains
testing is case insensitive:

cid = CaseInsensitiveDict()
cid['Accept'] = 'application/json'
cid['aCCEPT'] == 'application/json' # True
list(cid) == ['Accept'] # True

For example, ``headers['content-encoding']`` will return the
value of a ``'Content-Encoding'`` response header, regardless
of how the header name was originally stored.

If the constructor, ``.update``, or equality comparison
operations are given keys that have equal ``.lower()``s, the
behavior is undefined.

"""

    def __init__(self, data=None, **kwargs):
        self._store = dict()
        if data is None:
            data = {}
        self.update(data, **kwargs)

    def __setitem__(self, key, value):
        # Use the lowercased key for lookups, but store the actual
        # key alongside the value.
        self._store[key.lower()] = (key, value)

    def __getitem__(self, key):
        return self._store[key.lower()][1]

    def __delitem__(self, key):
        del self._store[key.lower()]

    def __iter__(self):
        return (casedkey for casedkey, mappedvalue in self._store.values())

    def __len__(self):
        return len(self._store)

    def lower_items(self):
        """Like iteritems(), but with all lowercase keys."""
        return (
            (lowerkey, keyval[1])
            for (lowerkey, keyval)
            in self._store.items()
        )

    def __eq__(self, other):
        if isinstance(other, collections.Mapping):
            other = CaseInsensitiveDict(other)
        else:
            return NotImplemented
        # Compare insensitively
        return dict(self.lower_items()) == dict(other.lower_items())

    # Copy is required
    def copy(self):
        return CaseInsensitiveDict(self._store.values())

    def __repr__(self):
        return '%s(%r)' % (self.__class__.__name__, dict(self.items()))



from floraconcierge.shortcuts import get_apiclient


def send_feedback(postObject):
    return get_apiclient().services.system.send_feedback(postObject)


def get_language_by_id(id):
    return get_apiclient().services.system.get_language_by_id(id)


def get_languages_list(limit=None, offset=None):
    return get_apiclient().services.system.get_languages_list(
        limit=limit,
        offset=offset)


def get_page_by_id(id):
    return get_apiclient().services.system.get_page_by_id(id)


def get_pages_list(category=None):
    return get_apiclient().services.system.get_pages_list(category=category)


def get_snippet_by_id(id):
    return get_apiclient().services.system.get_snippet_by_id(id)



#!/usr/bin/env python
import os

from setuptools import setup, find_packages


def read(filename):
    return open(os.path.join(os.path.dirname(__file__), filename)).read()


setup(
    name='floraconcierge-client',
    version=__import__('floraconcierge').get_version(),
    author='Alexandr Shurigin',
    author_email='alexandr.shurigin@gmail.com',
    description='FloraExpress API python client library. www.floraexpress.ru',
    license='BSD',
    keywords='floraconcierge floraexpress ecommerce api adapter client',
    url='http://www.floraexpress.ru/',
    packages=find_packages(exclude=['tests']),
    include_package_data=True,
    test_suite='tests',
    long_description=read("README.rst"),
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Customer Service',
        'Intended Audience :: Information Technology',
        'Intended Audience :: Legal Industry',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Natural Language :: Russian',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: PHP',
        'Topic :: Database :: Front-Ends',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Embedded Systems'
    ], install_requires=['pytz']
)


from django.contrib.auth.models import BaseUserManager, PermissionsMixin
from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _

from floraconcierge.mapping.model import User as ApiUser


class StaffManager(BaseUserManager):
    def _create_user(self, email, is_staff, is_superuser, **extra_fields):
        """
        Creates and saves a User with the given email and password.
        """
        now = timezone.now()
        if not email:
            raise ValueError('The given email must be set')

        email = self.normalize_email(email)
        user = self.model(email=email,
                          is_staff=is_staff, is_active=True,
                          is_superuser=is_superuser, last_login=now,
                          date_joined=now, **extra_fields)
        user.save(using=self._db)
        return user

    def create_user(self, email, **extra_fields):
        return self._create_user(email, False, False, **extra_fields)

    def create_superuser(self, email, password, **extra_fields):
        return self._create_user(email, True, True, **extra_fields)


class User(PermissionsMixin):
    email = models.EmailField(unique=True)
    date_joined = models.DateTimeField(_('date joined'), default=timezone.now)
    last_login = models.DateTimeField(_('last login'), default=timezone.now)

    is_staff = models.BooleanField(_('staff status'), default=False,
                                   help_text=_('Designates whether the user can log into this admin '
                                               'site.'))
    is_active = models.BooleanField(_('active'), default=True,
                                    help_text=_('Designates whether this user should be treated as '
                                                'active. Unselect this instead of deleting accounts.'))

    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = []

    objects = StaffManager()

    def __init__(self, *args, **kwargs):
        api_user_model = kwargs.pop('api_user_model', None)

        super(User, self).__init__(*args, **kwargs)
        self.info = api_user_model

    @classmethod
    def from_api_user(cls, user):
        assert isinstance(user, ApiUser)

        try:
            obj = cls.objects.get(email=user.email)
            obj.info = user
        except User.DoesNotExist:
            obj = cls(api_user_model=user)

        return obj

    def save(self, force_insert=False, force_update=False, using=None,
             update_fields=None):
        if self.id or not update_fields:
            return super(User, self).save(force_insert=force_insert, force_update=force_update, using=using,
                                          update_fields=update_fields)

    def get_email(self):
        return self.email or self.info.email

    def get_username(self):
        return self.get_email()

    def is_anonymous(self):
        return self.info is not None

    def is_authenticated(self):
        return bool(self.info)

    def __str__(self):
        return self.get_email()

    def __unicode__(self):
        return self.get_email()

    class Meta:
        verbose_name = _('user')
        verbose_name_plural = _('users')
        swappable = 'AUTH_USER_MODEL'



from floraconcierge.shortcuts import get_apiclient


def history_all_ref_codes(after_id=None):
    return get_apiclient().services.partner.history_all_ref_codes(
        after_id=after_id)


def check_ref_code(code):
    return get_apiclient().services.partner.check_ref_code(code)


def generate_ref_code():
    return get_apiclient().services.partner.generate_ref_code()


def history_ref_code(code, limit=None, offset=None):
    return get_apiclient().services.partner.history_ref_code(
        code,
        limit=limit,
        offset=offset)
